package com.tzld.piaoquan.recommend.server.service;

import com.tzld.piaoquan.recommend.server.gen.common.Result;
import com.tzld.piaoquan.recommend.server.gen.recommend.HomepageRecommendRequest;
import com.tzld.piaoquan.recommend.server.gen.recommend.HomepageRecommendResponse;
import com.tzld.piaoquan.recommend.server.gen.recommend.Video;
import com.tzld.piaoquan.recommend.server.model.param.HomepageRecommendParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author dyp
 */
@Service
public class RecommendService {
    @Autowired
    private MidService midService;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    private String abExpCode;

    private Map<String, String> ab_initial_config_map = new HashMap<>();

    @PostConstruct
    public void init() {
        ab_initial_config_map.put("VLOG", "095");
        ab_initial_config_map.put("LOVE_LIVE", "144");
        ab_initial_config_map.put("LONG_VIDEO", "121");
        ab_initial_config_map.put("LAO_HAO_KAN_VIDEO", "074");
        ab_initial_config_map.put("ZUI_JING_QI", "069");
        ab_initial_config_map.put("other", "095-1");
    }

    public HomepageRecommendResponse homepageRecommend(HomepageRecommendRequest request) {

        if (request.getVersionAuditStatus() == 1) {
            return specialMidRecommend(request);
        }
        if (StringUtils.isNotBlank(request.getMid())
                && redisTemplate.opsForSet().isMember("special:mid", request.getMid())) {
            return specialMidRecommend(request);
        }

        HomepageRecommendParam param = genHomepageRecommendParam(request);
        return null;

    }

    private HomepageRecommendParam genHomepageRecommendParam(HomepageRecommendRequest request) {
        HomepageRecommendParam param = new HomepageRecommendParam();
        param.setTop_K(3);
        param.setFlow_pool_P(0.3);
        param.setNo_op_flag(true);
        param.setExpire_time(3600);
        param.setOld_video_index(-1);

        String abInitialConfig = ab_initial_config_map.containsKey(request.getAppType())
                ? ab_initial_config_map.get(request.getAppType())
                : ab_initial_config_map.get("other");


        return param;
    }

    private HomepageRecommendResponse specialMidRecommend(HomepageRecommendRequest request) {
        String keyNamePrefix = "special:videos:item:";
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
        Date currentDate = new Date();
        String dateStr = dateFormat.format(currentDate);
        String specialKeyName = keyNamePrefix + dateStr;
        if (!redisTemplate.hasKey(specialKeyName)) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(currentDate);
            calendar.add(Calendar.DAY_OF_MONTH, -1);
            Date previousDate = calendar.getTime();
            dateStr = dateFormat.format(previousDate);
            specialKeyName = keyNamePrefix + dateStr;
        }

        String lastSpecialRecallKey = String.format("recall:last:special:%s:%s:%s", request.getAppType(), request.getMid(), dateStr);
        String value = redisTemplate.opsForValue().get(lastSpecialRecallKey);
        Long idx = 0L;
        if (StringUtils.isNotBlank(value)) {
            idx = redisTemplate.opsForZSet().reverseRank(specialKeyName, value);
            idx = idx == null ? 0L : idx;
        }

        int getSize = request.getSize() * 5;
        int freq = 0;
        List<Video> results = new ArrayList<>();
        while (results.size() < request.getSize()) {
            freq += 1;
            if (freq > 2) {
                break;
            }
            Set<ZSetOperations.TypedTuple<String>> data = redisTemplate.opsForZSet().reverseRangeWithScores(specialKeyName, idx, idx + getSize - 1);
            if (CollectionUtils.isEmpty(data)) {
                break;
            }
            idx += getSize;
            data.stream().forEach(t ->
                    results.add(Video.newBuilder()
                            .setVideoId(Long.getLong(t.getValue(), 0L))
                            .setRovScore(t.getScore())
                            .setAbCode("99999")
                            .setPushFrom("special_mid_videos")
                            .build())
            );
        }

        if (StringUtils.isNotBlank(request.getMid()) && !CollectionUtils.isEmpty(results)) {
            redisTemplate.opsForValue().set(lastSpecialRecallKey, String.valueOf(results.get(results.size() - 1).getVideoId()), 1, TimeUnit.DAYS);
        }

        return HomepageRecommendResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1).setMessage("success"))
                .addAllVideo(results)
                .build();
    }

}
